from player import Player


class Lineup:
    def __init__(self, players):
        '''
        :param players: List[Player]
        '''
        self.players = players

    def __str__(self):
        res = '\n'.join([str(index + 1) + ". " + str(player) for index, player in enumerate(self.players)])
        res += '\nFantasy Points ' + str(sum(player.fppg for player in self.players))
        res += '\nSalary ' + str(sum(player.salary for player in self.players))
        return res

    @property
    def lineup(self):
        '''
        :rtype: list[Player]
        '''
        return self.players

    @property
    def fantasy_points_projection(self):
        '''
        :rtype: int
        '''
        return sum(player.fppg for player in self.players)

    @property
    def salary_costs(self):
        '''
        :rtype: int
        '''
        return sum(player.salary for player in self.players)



__version__ = "0.2"

from .player import Player
from .exceptions import LineupOptimizerException
from .settings import *
from .lineup_optimizer import LineupOptimizer
from .app.app import run


def run_app():
    run()



class Player(object):
    def __init__(self, id,  first_name, last_name, position, team, salary, fppg, is_injured=False):
        self.id = id
        self.first_name = first_name
        self.last_name = last_name
        self.position = position
        self.team = team
        self.salary = salary
        self.fppg = fppg
        self.is_injured = is_injured

    def __str__(self):
        return "{}{}{}{}{}".format(
            "{:<30}".format(self.full_name),
            "{:<4}".format(self.position),
            "{:<15}".format(self.team),
            "{:<6}".format(str(self.fppg)),
            "{:<10}".format(str(self.salary) + '$')
        )

    @property
    def full_name(self):
        return "{} {}".format(self.first_name, self.last_name)

    @property
    def efficiency(self):
        return round(self.fppg / self.salary, 2)



import unittest
from pydfs_lineup_optimizer.lineup_optimizer import LineupOptimizer
from pydfs_lineup_optimizer.player import Player
from pydfs_lineup_optimizer import settings
from pydfs_lineup_optimizer.exceptions import LineupOptimizerException


class TestLineupOptimizer(unittest.TestCase):
    def setUp(self):
        self.lineup_optimizer = LineupOptimizer(settings.YahooBasketballSettings)
        self.lineup_optimizer.load_players_from_CSV("tests/yahoo_dfs_test_sample.csv")
        self._player1 = self.lineup_optimizer._players[0]

    def test_add_player_to_lineup(self):
        self.lineup_optimizer.reset_lineup()
        self.lineup_optimizer.add_player_to_lineup(self._player1)
        self.assertTrue(self._player1 in self.lineup_optimizer._lineup)

    def test_same_players_in_lineup(self):
        self.lineup_optimizer.reset_lineup()
        self.lineup_optimizer.add_player_to_lineup(self._player1)
        with self.assertRaises(LineupOptimizerException):
            self.lineup_optimizer.add_player_to_lineup(self._player1)

    def test_adding_player_with_salary_bigger_than_budget(self):
        self.lineup_optimizer.reset_lineup()
        player = Player('', '', '', 'PG', 'DEN', 100000, 2)
        with self.assertRaises(LineupOptimizerException):
            self.lineup_optimizer.add_player_to_lineup(player)

    def test_adding_player_to_formed_position(self):
        self.lineup_optimizer.reset_lineup()
        players = []
        for i in 'abcd':
            players.append(Player(i, i, i, 'PG', 'DEN', 10, 2))
        for i in range(3):
            self.lineup_optimizer.add_player_to_lineup(players[i])
        with self.assertRaises(LineupOptimizerException):
            self.lineup_optimizer.add_player_to_lineup(players[3])

    def test_remove_player_from_lineup(self):
        self.lineup_optimizer.reset_lineup()
        player1 = Player('', 'P', 'P', 'PG', 'DEN', 10, 2)
        player2 = Player('', 'C', 'C', 'PG', 'DEN', 10, 2)
        player3 = Player('', 'P', 'P', 'PG', 'DEN', 10, 2)
        self.lineup_optimizer.add_player_to_lineup(player1)
        self.lineup_optimizer.remove_player_from_lineup(player1)
        self.assertEqual(len(self.lineup_optimizer._lineup), 0)
        self.lineup_optimizer.add_player_to_lineup(player1)
        self.lineup_optimizer.add_player_to_lineup(player2)
        self.lineup_optimizer.add_player_to_lineup(player3)
        self.lineup_optimizer.remove_player_from_lineup(player1)
        self.assertEqual(self.lineup_optimizer._positions[('PG', )], 0)
        self.assertEqual(self.lineup_optimizer._positions[('PG', 'SG')], 1)
        self.assertEqual(self.lineup_optimizer._no_position_players, 1)
        self.lineup_optimizer.remove_player_from_lineup(player2)
        self.assertEqual(self.lineup_optimizer._positions[('PG', )], 0)
        self.assertEqual(self.lineup_optimizer._positions[('PG', 'SG')], 2)
        self.assertEqual(self.lineup_optimizer._no_position_players, 1)
        self.lineup_optimizer.remove_player_from_lineup(player3)
        self.assertEqual(self.lineup_optimizer._positions[('PG', )], 1)
        self.assertEqual(self.lineup_optimizer._positions[('PG', 'SG')], 3)
        self.assertEqual(self.lineup_optimizer._no_position_players, 1)

    def test_lineup_with_players_from_same_team(self):
        self.lineup_optimizer.reset_lineup()
        lineup = self.lineup_optimizer.optimize(teams={'LAL': 4, 'BOS': 4})[0]
        self.assertEqual(len(filter(lambda x: x.team == 'LAL', lineup.lineup)), 4)
        self.assertEqual(len(filter(lambda x: x.team == 'BOS', lineup.lineup)), 4)

    def test_lineup_with_players_from_same_positions(self):
        self.lineup_optimizer.reset_lineup()
        lineup = self.lineup_optimizer.optimize(positions={'PG': 3, 'SF': 2})[0]
        self.assertEqual(len(filter(lambda x: x.position == 'PG', lineup.lineup)), 3)
        self.assertEqual(len(filter(lambda x: x.position == 'SF', lineup.lineup)), 2)


def run_tests():
    unittest.main()


if __name__ == '__main__':
    run_tests()



from Tkinter import *
import ttk
import tkMessageBox, tkFileDialog
from pydfs_lineup_optimizer import *
from pydfs_lineup_optimizer.constants import *
from pydfs_lineup_optimizer.exceptions import LineupOptimizerException


LINEUPS_NUMBER = 10


class App(Tk):
    def __init__(self):
        Tk.__init__(self)
        self.title("pydfs-lineup-optimizer")
        self.minsize(1000, 600)
        self.resizable(True, True)
        self.optimizer = None
        self.lineups = []
        container = Frame(self)
        container.pack(side="top", fill="both", expand = True)
        container.grid_rowconfigure(0, weight=1)
        container.grid_columnconfigure(0, weight=1)
        self.frames = {}
        for F in (SelectLineupFrame, SelectOptimizerFrame, LoadPlayersFrame, PrintLineupFrame):
            frame = F(container, self)
            self.frames[F] = frame
            frame.grid(row=0, column=0, sticky="nsew")
        self.show_frame(SelectOptimizerFrame)

    def show_frame(self, cont):
        frame = self.frames[cont]
        if cont == SelectLineupFrame:
            frame.load_players()
        elif cont == PrintLineupFrame:
            frame.load_lineups()
        frame.tkraise()


class SelectOptimizerFrame(Frame):
    def __init__(self, master, controller):
        Frame.__init__(self, master)
        self.controller = controller
        self.create_widgets()
        self.place_widgets()

    def set_optimizer(self):
        try:
            site = self.dfs_sites_dropbox.get()
            sport = self.dfs_sports_dropbox.get()
            setting = DFS_SETTINGS_DICT[(site, sport)]
            self.controller.optimizer = LineupOptimizer(setting)
            self.controller.show_frame(LoadPlayersFrame)
        except KeyError:
            tkMessageBox.showerror("Error!", "Wrong sport or site!")

    def create_widgets(self):
        self.dfs_sites_dropbox = ttk.Combobox(self, values=DFS_SITES, state="readonly", height=5)
        self.dfs_sites_dropbox.current(0)
        self.dfs_sports_dropbox = ttk.Combobox(self, values=DFS_SPORTS, state="readonly", height=5)
        self.dfs_sports_dropbox.current(0)
        self.select_button = Button(self, text="Select", height=5, width=25, command=self.set_optimizer)

    def place_widgets(self):
        Label(self, text="SELECT DAILY FANTASY SPORT", font=(None, 28)).pack()
        self.dfs_sites_dropbox.place(relx=0.5, rely=0.4, anchor=CENTER)
        self.dfs_sports_dropbox.place(relx=0.5, rely=0.45, anchor=CENTER)
        self.select_button.place(relx=0.5, rely=0.6, anchor=CENTER)


class LoadPlayersFrame(Frame):
    def __init__(self, master, controller):
        Frame.__init__(self, master)
        self.controller = controller
        self.create_widgets()
        self.place_widgets()

    def create_widgets(self):
        Label(self, text="SELECT CSV FILE WITH PLAYERS", font=(None, 28)).pack()
        self.select_file_button = Button(self, text="Select file", command=self.open_file, height=5, width=20)

    def place_widgets(self):
        self.select_file_button.place(relx=0.5, rely=0.5, anchor=CENTER)

    def open_file(self):
        options = {}
        options['defaultextension'] = '.csv'
        options['filetypes'] = [('CSV files', '.csv'), ]
        options['parent'] = self
        options['title'] = 'Select CSV file with players'
        path = tkFileDialog.askopenfilename(**options)
        try:
            self.controller.optimizer.load_players_from_CSV(path)
            self.controller.show_frame(SelectLineupFrame)
        except Exception as e:
            tkMessageBox.showerror("Error!", e.message)


class SelectLineupFrame(Frame):
    def __init__(self, master, controller):
        Frame.__init__(self, master)
        self.controller = controller
        self.create_widgets()
        self.place_widgets()

    def create_widgets(self):
        self.info = StringVar()
        self.info_label = Label(self, textvariable=self.info, font=(None, 16), borderwidth=5)
        self.all_players_table = ttk.Treeview(self)
        self.all_players_table.tag_configure('injured', background='red')
        self.all_players_table['show'] = 'headings'
        self.all_players_table["columns"] = ("Name", "Team", "Position", "FPPG", "Salary", "Efficiency")
        self.all_players_table.column("Name", width=200)
        self.all_players_table.column("Team", width=60)
        self.all_players_table.column("Position", width=60)
        self.all_players_table.column("FPPG", width=60)
        self.all_players_table.column("Salary", width=60)
        self.all_players_table.column("Efficiency", width=80)
        self.all_players_table.heading("Name", text="Name")
        self.all_players_table.heading("Team", text="Team")
        self.all_players_table.heading("Position", text="Position")
        self.all_players_table.heading("FPPG", text="FPPG")
        self.all_players_table.heading("Salary", text="Salary")
        self.all_players_table.heading("Efficiency", text="Efficiency")

        self.remove_player_button = Button(self, text='Remove', command=self.remove_player)
        self.return_player_button = Button(self, text='Restore', command=self.restore_player)

        self.removed_players_table = ttk.Treeview(self)
        self.removed_players_table.tag_configure('injured', background='red')
        self.removed_players_table['show'] = 'headings'
        self.removed_players_table["columns"] = ("Name", "Team", "Position", "FPPG", "Salary", "Efficiency")
        self.removed_players_table.column("Name", width=200)
        self.removed_players_table.column("Team", width=60)
        self.removed_players_table.column("Position", width=60)
        self.removed_players_table.column("FPPG", width=60)
        self.removed_players_table.column("Salary", width=60)
        self.removed_players_table.column("Efficiency", width=80)
        self.removed_players_table.heading("Name", text="Name")
        self.removed_players_table.heading("Team", text="Team")
        self.removed_players_table.heading("Position", text="Position")
        self.removed_players_table.heading("FPPG", text="FPPG")
        self.removed_players_table.heading("Salary", text="Salary")
        self.removed_players_table.heading("Efficiency", text="Efficiency")

        self.add_to_lineup_button = Button(self, text='Add to lineup', command=self.add_to_lineup)
        self.remove_from_lineup_button = Button(self, text='Remove from lineup', command=self.remove_from_lineup)

        self.lineup_table = ttk.Treeview(self)
        self.lineup_table.tag_configure('injured', background='red')
        self.lineup_table['show'] = 'headings'
        self.lineup_table["columns"] = ("Name", "Team", "Position", "FPPG", "Salary", "Efficiency")
        self.lineup_table.column("Name", width=200)
        self.lineup_table.column("Team", width=60)
        self.lineup_table.column("Position", width=60)
        self.lineup_table.column("FPPG", width=60)
        self.lineup_table.column("Salary", width=60)
        self.lineup_table.column("Efficiency", width=80)
        self.lineup_table.heading("Name", text="Name")
        self.lineup_table.heading("Team", text="Team")
        self.lineup_table.heading("Position", text="Position")
        self.lineup_table.heading("FPPG", text="FPPG")
        self.lineup_table.heading("Salary", text="Salary")
        self.lineup_table.heading("Efficiency", text="Efficiency")

        self.generate_lineup_button = Button(self, text='Generate lineup', command=self.generate_lineup)
        self.reset_lineup_button = Button(self, text='Clear lineup', command=self.reset_lineup)
        self.with_injured = IntVar()
        self.with_injured_checkbox = Checkbutton(self, text="Use injured players", variable=self.with_injured)

    def place_widgets(self):
        self.info_label.grid(row=0, column=0)
        Label(self, text='All Players').grid(row=1, column=0)
        Label(self, text='Removed PLayers').grid(row=1, column=2)
        self.all_players_table.grid(row=2, column=0, rowspan=5)
        self.remove_player_button.grid(row=3, column=1)
        self.return_player_button.grid(row=4, column=1)
        self.removed_players_table.grid(row=2, column=2, rowspan=5)
        self.add_to_lineup_button.grid(row=9, column=0)
        self.remove_from_lineup_button.grid(row=11, column=0)
        Label(self, text='Lineup').grid(row=13, column=0)
        self.lineup_table.grid(row=14, column=0, rowspan=5)
        self.with_injured_checkbox.grid(row=17, column=2)
        self.generate_lineup_button.grid(row=15, column=2)
        self.reset_lineup_button.grid(row=16, column=2)

    def remove_player(self):
        try:
            item = self.all_players_table.selection()[0]
            index = self.all_players_table.index(item)
            player = self.controller.optimizer.players[index]
            self.controller.optimizer.remove_player(player)
            self.load_players()
        except IndexError:
            pass

    def restore_player(self):
        try:
            item = self.removed_players_table.selection()[0]
            index = self.removed_players_table.index(item)
            player = self.controller.optimizer.removed_players[index]
            self.controller.optimizer.restore_player(player)
            self.load_players()
        except IndexError:
            pass

    def generate_lineup(self):
        with_injured = self.with_injured.get()
        self.controller.lineups = self.controller.optimizer.optimize(LINEUPS_NUMBER, with_injured=with_injured)
        self.controller.show_frame(PrintLineupFrame)

    def reset_lineup(self):
        self.controller.optimizer.reset_lineup()
        self.load_players()

    def add_to_lineup(self):
        try:
            try:
                item = self.all_players_table.selection()[0]
            except IndexError:
                return
            index = self.all_players_table.index(item)
            player = self.controller.optimizer.players[index]
            self.controller.optimizer.add_player_to_lineup(player)
            self.load_players()
        except LineupOptimizerException as e:
            tkMessageBox.showerror("Error!", e.message)

    def remove_from_lineup(self):
        try:
            try:
                item = self.lineup_table.selection()[0]
            except IndexError:
                return
            index = self.lineup_table.index(item)
            player = self.controller.optimizer.lineup[index]
            self.controller.optimizer.remove_player_from_lineup(player)
            self.load_players()
        except LineupOptimizerException as e:
            tkMessageBox.showerror("Error!", e.message)

    def load_players(self):
        self.all_players_table.delete(*self.all_players_table.get_children())
        self.removed_players_table.delete(*self.removed_players_table.get_children())
        self.lineup_table.delete(*self.lineup_table.get_children())
        for index, player in enumerate(self.controller.optimizer.players):
            self.all_players_table.insert("", index, text='', tags=('injured' if player.is_injured else None, ),
                                          values=(player.full_name, player.team,
                                                  player.position, player.fppg, player.salary, player.efficiency))
        for index, player in enumerate(self.controller.optimizer.removed_players):
            self.removed_players_table.insert("", index, text='', tags=('injured' if player.is_injured else None, ),
                                              values=(player.full_name, player.team,
                                                      player.position, player.fppg, player.salary, player.efficiency))
        for index, player in enumerate(self.controller.optimizer.lineup):
            self.lineup_table.insert("", index, text='', tags=('injured' if player.is_injured else None, ),
                                     values=(player.full_name, player.team,
                                             player.position, player.fppg, player.salary, player.efficiency))


class PrintLineupFrame(Frame):
    def __init__(self, master, controller):
        Frame.__init__(self, master)
        self.controller = controller
        self.create_widgets()
        self.place_widgets()

    def create_widgets(self):
        self.scrollbar = Scrollbar(self)
        self.text = Text(self, wrap=WORD, yscrollcommand=self.scrollbar.set)
        self.scrollbar.config(command=self.text.yview)
        self.return_button = Button(self, text='Return', command=self.return_to_selecting)

    def place_widgets(self):
        self.return_button.pack()
        self.scrollbar.pack(side=RIGHT, fill=Y)
        self.text.pack(fill=BOTH)

    def load_lineups(self):
        lineups = "\n\n".join(["N {}\n{}".format(index + 1, str(lineup)) for index, lineup in enumerate(self.controller.lineups)])
        self.text.insert(INSERT, lineups)

    def return_to_selecting(self):
        self.controller.show_frame(SelectLineupFrame)


def run():
    root = App()
    root.mainloop()

if __name__ == '__main__':
    run()



from pulp import *
from exceptions import LineupOptimizerException
from settings import BaseSettings
from player import Player
from lineup import Lineup


class LineupOptimizer(object):
    def __init__(self, settings):
        '''
        LineupOptimizer select the best lineup for daily fantasy sports.
        :type settings: BaseSettings
        '''
        self._players = []
        self._lineup = []
        self._set_settings(settings)
        self._removed_players = []

    @property
    def lineup(self):
        '''
        :rtype: list[Player]
        '''
        return self._lineup

    @property
    def budget(self):
        '''
        :rtype: int
        '''
        return self._budget

    @property
    def players(self):
        '''
        :rtype: list[Player]
        '''
        return [player for player in self._players if player not in self._removed_players and player not in self._lineup]

    @property
    def removed_players(self):
        '''
        :rtype: list[Player]
        '''
        return self._removed_players

    def _set_settings(self, settings=None):
        '''
        Set settings with daily fantasy sport site and kind of sport to optimizer.
        :type settings: BaseSettings
        '''
        if settings is not None:
            self._settings = settings
        self._budget = self._settings.budget
        self._total_players = self._settings.total_players
        self._positions = self._settings.positions.copy()
        self._no_position_players = self._settings.no_position_players

    def reset_lineup(self):
        '''
        Reset current lineup.
        '''
        self._set_settings()
        self._lineup = []

    def load_players_from_CSV(self, filename):
        '''
        Load player list from CSV file with passed filename.
        Calls load_players_from_CSV method from _settings object.
        :type filename: str
        '''
        self._players = self._settings.load_players_from_CSV(filename)

    def remove_player(self, player):
        '''
        Remove player from list for selecting players for lineup.
        :type player: Player
        '''
        self._removed_players.append(player)

    def restore_player(self, player):
        try:
            self._removed_players.remove(player)
        except ValueError:
            pass

    def _add_to_lineup(self, player):
        '''
        Adding player to lineup without checks
        :type player: Player
        '''
        self._lineup.append(player)
        self._total_players -= 1
        self._budget -= player.salary

    def add_player_to_lineup(self, player):
        '''
        Force adding specified player to lineup.
        Return true if player successfully added to lineup.
        :type player: Player
        '''
        if player in self._lineup:
            raise LineupOptimizerException("This player already in your line up!")
        if not isinstance(player, Player):
            raise LineupOptimizerException("This function accept only Player objects!")
        try:
            if self._budget - player.salary < 0:
                raise LineupOptimizerException("Can't add this player to line up! Your team is over budget!")
            if self._total_players - 1 < 0:
                raise LineupOptimizerException("Can't add this player to line up! You already select all {} players!".
                                               format(len(self._lineup)))
            try:
                is_added = False

                def decrement_player_from_all_positions():
                    is_added = False
                    for key, value in self._positions.items():
                        if player.position in key and value >= 1:
                            self._positions[key] -= 1
                            is_added = True
                    return is_added

                try:
                    if self._positions[(player.position, )] > 0:
                        is_added = decrement_player_from_all_positions()
                    else:
                        for key, value in self._positions.items():
                            if player.position in key and len(key) > 1:
                                for pos in key:
                                    if pos != player.position:
                                        try:
                                            value -= self._positions[(pos, )]
                                        except KeyError:
                                            pass
                                if value:
                                    is_added = decrement_player_from_all_positions()
                except KeyError:
                    pass
                if not is_added and self._no_position_players:
                    self._no_position_players -= 1
                    is_added = True
                if is_added:
                    self._add_to_lineup(player)
                else:
                    raise LineupOptimizerException("You're already select all {}'s".format(player.position))
            except KeyError:
                raise LineupOptimizerException("This player has wrong position!")
        except ValueError:
            raise LineupOptimizerException("Player not in players list!")

    def remove_player_from_lineup(self, player):
        '''
        Remove specified player from lineup.
        :type player: Player
        '''
        if not isinstance(player, Player):
            raise LineupOptimizerException("This function accept only Player objects!")
        try:
            occupied_positions = {key: len(filter(lambda player: player.position in key, self._lineup))
                                  for key in self._positions.keys()}
            positions_difference = {key: occupied_positions[key] - self._settings.positions[key]
                                    for key in occupied_positions.keys()}
            try:
                if positions_difference[(player.position, )] == 0:
                    for key in self._positions.keys():
                        if player.position in key:
                            self._positions[key] += 1
                else:
                    is_no_position_player = True
                    for key, value in positions_difference.items():
                        if player.position in key and value < 0:
                            is_no_position_player = False
                    if is_no_position_player:
                        self._no_position_players += 1
                    else:
                        for key in self._positions.keys():
                            if player.position in key and key != (player.position, ):
                                self._positions[key] += 1
                self._lineup.remove(player)
                self._budget += player.salary
                self._total_players += 1
            except KeyError:
                pass
        except ValueError:
            raise LineupOptimizerException("Player not in line up!")

    def optimize(self, n=5,  teams=None, positions=None, with_injured=False):
        '''
        Select optimal lineup from players list.
        This method uses Mixed Integer Linear Programming method for evaluating best starting lineup.
        It's return list with n lineup object. This list sorted by
        :type n: int
        :type teams: dict[str, int]
        :type positions: dict[str, int]
        :type with_injured: bool
        :rtype: List[Lineup]
        '''
        lineups = []
        current_max_points = 100000
        for i in range(n):
            players = [player for player in self._players if player not in self._removed_players and player not in self._lineup and isinstance(player, Player)]
            prob = LpProblem("Daily Fantasy Sports", LpMaximize)
            x = LpVariable.dicts(
                'table', players,
                lowBound = 0,
                upBound = 1,
                cat = LpInteger
            )
            prob += sum([player.fppg * x[player] for player in players])
            prob += sum([player.fppg * x[player] for player in players]) <= current_max_points
            prob += sum([player.salary * x[player] for player in players]) <= self._budget
            prob += sum([x[player] for player in players]) == self._total_players
            if not with_injured:
                prob += sum([x[player] for player in players if not player.is_injured]) == self._total_players
            for position, num in self._positions.items():
                prob += sum([x[player] for player in players if player.position in position]) >= num
            if teams is not None:
                for key, value in teams.items():
                    prob += sum([x[player] for player in players if player.team == key]) == value
            if positions is not None:
                for key, value in positions.items():
                    prob += sum([x[player] for player in players if player.position == key]) == value
            prob.solve()
            if prob.status == 1:
                lineup_players = self._lineup[:]
                for player in players:
                    if x[player].value() == 1.0:
                        lineup_players.append(player)
                lineup = Lineup(lineup_players)
                current_max_points = lineup.fantasy_points_projection - 0.1
                lineups.append(lineup)
            else:
                raise LineupOptimizerException("Can't create optimal lineup! Wrong input data!")
        return lineups



from distutils.core import setup

setup(
    name = 'pydfs-lineup-optimizer',
    version = '0.3',
    packages = ['tests', 'pydfs_lineup_optimizer', 'pydfs_lineup_optimizer.app'],
    url = 'https://github.com/DimaKudosh/pydfs-lineup-optimizer',
    license = '',
    author = 'Dima Kudosh',
    author_email = 'dimakudosh@gmail.com',
    description = 'Tool for creating optimal lineups for daily fantasy sports',
    keywords = ['dfs', 'fantasy', 'sport', 'lineup', 'optimize', 'optimizer'],
    install_requires = ['PuLP'],
    classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'Operating System :: POSIX',
          'Programming Language :: Python',

          ],
)



"""
Store classes with settings for specified daily fantasy sports site and kind of sport.
"""
from abc import ABCMeta, abstractmethod
import csv
from player import Player


class BaseSettings(object):
    __metaclass__ = ABCMeta
    budget = 0
    total_players = 0
    no_position_players = 0
    positions = {}

    @abstractmethod
    def load_players_from_CSV(self, filename):
        return NotImplemented


# YAHOO


class YahooSettings(BaseSettings):
    @classmethod
    @abstractmethod
    def load_players_from_CSV(cls, filename):
        players = []
        with open(filename, 'r') as csvfile:
            csvdata = csv.DictReader(csvfile, skipinitialspace=True)
            for row in csvdata:
                player = Player(
                    row["Id"],
                    row["First Name"],
                    row["Last Name"],
                    row["Position"],
                    row["Team"],
                    float(row["Salary"]),
                    float(row["FPPG"]),
                    True if row["Injury Status"].strip() else False
                )
                players.append(player)
        return players


class YahooBasketballSettings(YahooSettings):
    budget = 200
    total_players = 8
    no_position_players = 1
    positions = {
        ('PG', ): 1,
        ('SG', ): 1,
        ('SF', ): 1,
        ('PF', ): 1,
        ('C', ): 1,
        ('PG', 'SG'): 3,
        ('PF', 'SF'): 3,
    }


class YahooFootballSettings(YahooSettings):
    budget = 200
    total_players = 9
    positions = {
        ('QB', ): 1,
        ('WR', ): 3,
        ('RB', ): 2,
        ('TE', ): 1,
        ('WR', 'RB', 'TE'): 7,
        ('DEF', ): 1,
    }


class YahooHockeySettings(YahooSettings):
    budget = 200
    total_players = 9
    positions = {
        ('G', ): 2,
        ('C', ): 2,
        ('LW', 'RW'): 3,
        ('D', ): 2,
    }


# FanDuel


class FanDuelSettings(BaseSettings):
    @classmethod
    @abstractmethod
    def load_players_from_CSV(cls, filename):
        players = []
        with open(filename, 'r') as csvfile:
            csvdata = csv.DictReader(csvfile, skipinitialspace=True)
            for row in csvdata:
                player = Player(
                    row["Id"],
                    row["First Name"],
                    row["Last Name"],
                    row["Position"],
                    row["Team"],
                    float(row["Salary"]),
                    float(row["FPPG"]),
                    True if row["Injury Indicator"].strip() else False
                )
                players.append(player)
        return players



class FanDuelBasketballSettings(FanDuelSettings):
    budget = 60000
    total_players = 9
    positions = {
        ('PG', ): 2,
        ('SG', ): 2,
        ('SF', ): 2,
        ('PF', ): 2,
        ('C', ): 1
    }


class FanDuelFootballSettings(FanDuelSettings):
    budget = 60000
    total_players = 9
    positions = {
        ('QB', ): 1,
        ('RB', ): 2,
        ('WR', ): 3,
        ('TE', ): 1,
        ('DF', ): 1,
        ('K', ): 1
    }


# DraftKings


class DraftKingsSettings(BaseSettings):
    @classmethod
    @abstractmethod
    def load_players_from_CSV(cls, filename):
        players = []
        with open(filename, 'r') as csvfile:
            csvdata = csv.DictReader(csvfile, skipinitialspace=True)
            for row in csvdata:
                name = row["Name"].split()
                player = Player(
                    '',
                    name[0],
                    name[1],
                    row["Position"],
                    '',
                    float(row["Salary"]),
                    float(row["AvgPointsPerGame"])
                )
                players.append(player)
        return players


class DraftKingsBasketballSettings(DraftKingsSettings):
    budget = 50000
    total_players = 8
    no_position_players = 1
    positions = {
        ('PG', ): 1,
        ('SG', ): 1,
        ('SF', ): 1,
        ('PF',): 1,
        ('C', ): 1,
        ('PG', 'SG'): 3,
        ('SF', 'PF'): 3
    }


class DraftKingsFootballSettings(DraftKingsSettings):
    budget = 50000
    total_players = 9
    positions = {
        ('QB', ): 1,
        ('RB', ): 2,
        ('WR', ): 3,
        ('TE', ): 1,
        ('RB', 'WR', 'TE'): 7,
        ('DST', ): 1
    }

# FantasyDraft


class FantasyDraftSettings(BaseSettings):
    @classmethod
    @abstractmethod
    def load_players_from_CSV(cls, filename):
        players = []
        with open(filename, 'r') as csvfile:
            csvdata = csv.DictReader(csvfile, skipinitialspace=True)
            for row in csvdata:
                name = row["Name"].split()
                player = Player(
                    "",
                    name[0],
                    name[1],
                    row["Position"],
                    row["Team"],
                    float(row["Salary"].replace('$', '').replace(',', '')),
                    float(row["Avg FPPG"])
                )
                players.append(player)
        return players


class FantasyDraftBasketballSettings(FantasyDraftSettings):
    budget = 100000
    total_players = 8
    no_position_players = 2
    positions = {
        ('PG', 'SG'): 3,
        ('PF', 'SF', 'C'): 3,
    }


class FantasyDraftFootballSettings(FantasyDraftSettings):
    budget = 100000
    total_players = 9
    no_position_players = 0
    positions = {
        ('QB', ): 1,
        ('RB', ): 2,
        ('WR', ): 2,
        ('TE', ): 1,
        ('RB', 'WR', 'TE'): 7,
        ('DST', ): 1
    }


class FantasyDraftHockeySettings(FantasyDraftSettings):
    budget = 100000
    total_players = 8
    no_position_players = 0
    positions = {
        ('C', ): 2,
        ('W', ): 2,
        ('D', ): 1,
        ('C', 'W', 'D'): 7,
        ('TG', ): 1,
    }


# Fanaments


class FanamentsSettings(BaseSettings):
    @classmethod
    @abstractmethod
    def load_players_from_CSV(cls, filename):
        players = []
        with open(filename, 'r') as csvfile:
            csvdata = csv.DictReader(csvfile, skipinitialspace=True)
            for row in csvdata:
                try:
                    player = Player(
                        row["Id"],
                        row["First Name"],
                        row["Last Name"],
                        row["Position"],
                        row["Team"],
                        float(row["Salary"]),
                        float(row["FPPG"]),
                        True if row["Injury Status"].strip() else False
                    )
                except Exception as e:
                    print row
                players.append(player)
        return players



from pydfs_lineup_optimizer.settings import *


DFS_SITES = ["Yahoo", "FanDuel", "DraftKings", "FantasyDraft"]
DFS_SPORTS = ["Basketball", "Football", "Hockey"]
DFS_SETTINGS_DICT = {
    ("Yahoo", "Basketball"): YahooBasketballSettings,
    ("Yahoo", "Football"): YahooFootballSettings,
    ("Yahoo", "Hockey"): YahooHockeySettings,
    ("FanDuel", "Basketball"): FanDuelBasketballSettings,
    ("FanDuel", "Football"): FanDuelFootballSettings,
    ("DraftKings", "Basketball"): DraftKingsBasketballSettings,
    ("DraftKings", "Football"): DraftKingsFootballSettings,
    ("FantasyDraft", "Basketball"): FantasyDraftBasketballSettings,
    ("FantasyDraft", "Football"): FantasyDraftFootballSettings,
    ("FantasyDraft", "Hockey"): FantasyDraftHockeySettings,
}